@@ -9,7 +9,6 @@ import qualified Test.Copilot.Core.External
 import qualified Test.Copilot.Core.Interpret.Eval
 import qualified Test.Copilot.Core.Type
 import qualified Test.Copilot.Core.Type.Array
-import qualified Test.Copilot.Core.Type.Dynamic
 import qualified Test.Copilot.Core.Type.Show
 
 -- | Run all unit tests on copilot-core.
@@ -23,6 +22,5 @@ tests =
   , Test.Copilot.Core.Interpret.Eval.tests
   , Test.Copilot.Core.Type.tests
   , Test.Copilot.Core.Type.Array.tests
-  , Test.Copilot.Core.Type.Dynamic.tests
   , Test.Copilot.Core.Type.Show.tests
   ]
